% J. Rowland Adams, A. Stefanovska and P. Clemson
% Example: integrate_network('AAutCoupling')
% To simulate a network, first use the generateparameters() function to generate a file with the specified parameters
function integrate_network(name)

    parameters = importdata(strcat(name,'_parameters.mat'));

    a=length(parameters);
    
    % vectorised functions
    dtheta_A = @(theta_A,K_A,N,omega_A) omega_A+(1/N)*sum(K_A.*sin(theta_A-theta_A'))';
   
    wb=waitbar(0,'Integrating...');
    
    for m = 1:a
     
        K_A = parameters(m).K_A;    % coupling strength between oscillatiors in nework A
        M_A = parameters(m).M_A;    % amplitude of coupling modulation in network A
        omega_A0 = parameters(m).omega_A0;    % base frequency of each oscillator in nework A
        omega_mA = parameters(m).omega_mA;    % frequency of coupling modulation in nework A

        N = parameters(m).N;    % number of oscillators in each network
        tf = parameters(m).tf;    % final time
        ts = parameters(m).ts;    % time between samples (integration step)
    
        time=0:ts:tf; %time span (double sampling frequency for intermediate RK steps)
        num_samples = tf/ts;

        % pre-compute external variables
        omega_A=freqmod(omega_A0,M_A,omega_mA,time);
        Omega_A = sum(omega_A,1)/N;

        results = struct();
		
		% initial values (deterministic system so these are the only
		% source of variation between runs)
		% uniformly samples on interval [0,2pi]
		theta_A = zeros(N,num_samples+1);
		theta_A(:,1) = 2*pi*rand(N,1);

		grad_theta_A = zeros(N,num_samples+1);

		for k = 1:num_samples
			% step 1
			k1_A = dtheta_A(theta_A(:,k),K_A,N,freqmod(omega_A0,M_A,omega_mA,time(k)));

			% step 2
			k2_A = dtheta_A(theta_A(:,k)+ts*k1_A/2,K_A,N,freqmod(omega_A0,M_A,omega_mA,time(k)+ts/2));

			% step 3
			k3_A = dtheta_A(theta_A(:,k)+ts*k2_A/2,K_A,N,freqmod(omega_A0,M_A,omega_mA,time(k)+ts/2));

			% step 4
			k4_A = dtheta_A(theta_A(:,k)+ts*k3_A,K_A,N,freqmod(omega_A0,M_A,omega_mA,time(k)+ts));

			theta_A(:,k+1) = theta_A(:,k)+(ts/6)*(k1_A+2*k2_A+2*k3_A+k4_A);
			grad_theta_A(:,k) = k1_A;

		end
		
		% one last integration step to get the final gradients
		k1_A = dtheta_A(theta_A(:,end),K_A,N,freqmod(omega_A0,M_A,omega_mA,time(end)));

		grad_theta_A(:,end) = k1_A;

		results.theta_A = theta_A;
		results.grad_theta_A = grad_theta_A;

        save(strcat(name,"_integration_",num2str(m),'.mat'),'results','omega_A','Omega_A', '-v7.3');

        waitbar(m/a);
    end

    close(wb);
end

function [result] = freqmod(omega_A0,M_A,omega_mA,time)
    result = omega_A0+omega_A0*M_A.*sin(omega_mA*time);
end
